#!/bin/bash

FAILED=0

cleanup () {
	rm -Rf ${SCRATCHDIR} 2>/dev/null
	mkdir ${SCRATCHDIR} 2>/dev/null
	rm -Rf ${TESTINGDIR}/outputs 2>/dev/null
	mkdir ${TESTINGDIR}/outputs -p 2>/dev/null
}

kill_podman () {
	ps -aux | grep podman | grep "$SCRATCHDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill $pid 2>/dev/null ; done
	ps -aux | grep podman | grep "$SCRATCHDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill -9 $pid 2>/dev/null ; done
	ps -aux | grep podmanwrapper | grep "$TESTDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill $pid 2>/dev/null ; done
	ps -aux | grep podmanwrapper | grep "$TESTDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill -9 $pid 2>/dev/null ; done
	ps -aux | grep "podman-real$" | grep "$TESTDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill $pid 2>/dev/null ; done
	ps -aux | grep "podman-real$" | grep "$TESTDIR" | grep "$USER" | sed 's/^[^ ]* *\([^ ]*\).*/\1/' | while read pid ; do kill -9 $pid 2>/dev/null ; done
}

failed () {
	if [ -z "$1" ] ; then
		echo "TEST FAILED"
	else
		echo "TEST $1 FAILED - ${2}"
	fi
	kill_podman
	cleanup
	FAILED=1
}

success () {
	if [ -n "$1" ] ; then
		echo "TEST $1 SUCCEEDED"
	else
		echo "TEST SUCCEDED"
	fi
	kill_podman
	cleanup
}

success_but_at_what_cost () {
	success "$1"
	echo "but it's not encrypted :/"
}

testpod () {
	if [ -z "$1" ] ; then
		failed "none" "didn't provide a testing suite"
		return
	fi
	cd "${1}"
	export TESTINGDIR=$(pwd)
	export SCRATCHDIR="${BACKUPSCRATCH}/${1}"
	export TMPDIR=$SCRATCHDIR
	mkdir "${TESTINGDIR}/outputs" -p
    mkdir "$SCRATCHDIR" -p
	touch "/tmp/${1}_testing_output"
	touch "/tmp/${1}_testing_error"


    if [ -z "${3}" ] ; then
        $(realpath "../test.sh") >"/tmp/${1}_testing_output" 2>"/tmp/${1}_testing_error" &
    else
        $(realpath "${3}") >"/tmp/${1}_testing_output" 2>"/tmp/${1}_testing_error" &
    fi

	sleep 1m

    ITERCOUNT=0
	while [ -z "$(grep "Script starting" "/tmp/${1}_testing_output")" ] ; do
		if [ $(ps -aux | grep "podman-real.*${1}" --color | sed 's/.*grep.*//' | sed 's/.*conmon.*//' | sed -r '/^\s*$/d' | wc -l) -lt 1 ] ; then
			failed "$1" "Script hasn't started and podman isn't running anymore"
			cd ..
			return
		fi
		sleep 30s
        ITERCOUNT=$((ITERCOUNT+1))
        if [ $ITERCOUNT -gt 10 ] ; then
			failed "$1" "Script hasn't started after 5 minutes, aborting"
			cd ..
            return
        fi
	done

	if [ -z "$(grep "TurboVNC.*started" "/tmp/${1}_testing_output")" ] && [ -z "${3}" ] ; then
		failed "$1" "Script started, but VNC didn't"
		cd ..
		return
	fi

    ITERCOUNT=0
	while [ -z "$(grep "${2}" "/tmp/${1}_testing_output")" ] ; do
		sleep 30s
		if [ $(ps -aux | grep "podman-real.*${1}" --color | sed 's/.*grep.*//' | sed 's/.*conmon.*//' | sed -r '/^\s*$/d' | wc -l) -lt 1 ] ; then
			cat "/tmp/${1}_testing_output"
			echo "${2}"
			failed "$1" "script.sh hasn't launched required program yet, but podman isn't running anymore"
			cd ..
			return
		fi
		if [ -n "$(grep -i "Segmentation *fault" "/tmp/${1}_testing_output")" ] ; then
			failed "$1" "script.sh caused a segmentation fault"
			cd ..
			return
		fi
        ITERCOUNT=$((ITERCOUNT+1))
        if [ $ITERCOUNT -gt 10 ] ; then
			failed "$1" "Hasn't found '${2}' within 5 minutes, aborting"
			cd ..
            return
        fi
	done

	# see if the pod survives for 5m
	sleep 5m

	if [ $(ps -aux | grep "podman-real.*${1}" | sed 's/.*grep.*//' | sed 's/.*conmon.*//' | sed -r '/^\s*$/d' | wc -l) -lt 1 ] ; then
		failed "$1" "pod died withing 5 minutes of script.sh execution"
		cd ..
		return
	fi

	if [ -n "$(grep -i "Segmentation *fault" "/tmp/${1}_testing_output")" ] ; then
		failed "$1" "pod is running, but there was a segmentation fault"
		cd ..
		return
	fi

	if [ -z "$(grep "No SSL/TLS support" "/tmp/${1}_testing_output")" ] ; then
		success "$1"
	else
		success_but_at_what_cost "$1"
	fi

	cd ..
}

echo "TESTING STARTED"

export IMAGE="cerit/debian10-xfce-gl"
export CUDA_VISIBLE_DEVICES=all
export BACKUPSCRATCH=$SCRATCHDIR

#-------------------Desktop-----------------------------
testpod "desktop" "Launching desktop.*xfce"

#-------------------ANSYS-------------------------------
testpod "ansys" "Launching desktop"

#-------------------ANSYS-------------------------------
testpod "matlab" "Launching matlab"

#-------------------VMD---------------------------------
testpod "vmd" "Launching VMD"

#-------------------Rstudio-----------------------------
testpod "rstudio" "\[services.d\] done" "../vncless.sh"

#-------------------End---------------------------------
echo "TESTING FINISHED"

exit $FAILED
